
/*********************************************************************
* Copyright (c) 2017 Tecnalia.
*
* This program and the accompanying materials are made
* available under the terms of the Eclipse Public License 2.0
* which is available at https://www.eclipse.org/legal/epl-2.0/
*
* SPDX-License-Identifier: EPL-2.0
* Contributors (in alphabetical order):
* Gorka Benguria                            Tecnalia
* Iaki Etxaniz                             Tecnalia
* Juncal Alonso                             Tecnalia
* Leire Orue-Echevarria                     Tecnalia
* Maria Jose Lopez                          Tecnalia
* Marisa Escalante                          Tecnalia
* Initially developed in the context of DECIDE EU project www.DECIDE-h2020.eu
**********************************************************************/
package eu.DECIDEh2020.optimus.editors;


import java.io.BufferedReader;
import java.io.FileNotFoundException;
import java.io.FileReader;
import java.io.IOException;
import java.io.StringWriter;
import java.text.Collator;
import java.util.List;
import java.util.ArrayList;
import java.util.Collections;
import java.util.StringTokenizer;

import org.eclipse.core.resources.IMarker;
import org.eclipse.core.resources.IResourceChangeEvent;
import org.eclipse.core.resources.IResourceChangeListener;
import org.eclipse.core.resources.ResourcesPlugin;
import org.eclipse.core.runtime.IProgressMonitor;
import org.eclipse.jface.dialogs.ErrorDialog;
import org.eclipse.swt.SWT;
import org.eclipse.swt.custom.StyledText;
import org.eclipse.swt.events.SelectionAdapter;
import org.eclipse.swt.events.SelectionEvent;
import org.eclipse.swt.graphics.Font;
import org.eclipse.swt.graphics.FontData;
import org.eclipse.swt.layout.FillLayout;
import org.eclipse.swt.layout.GridData;
import org.eclipse.swt.layout.GridLayout;
import org.eclipse.swt.widgets.Button;
import org.eclipse.swt.widgets.Composite;
import org.eclipse.swt.widgets.Display;
import org.eclipse.swt.widgets.FontDialog;
import org.eclipse.ui.*;
import org.eclipse.ui.editors.text.TextEditor;
import org.eclipse.ui.part.FileEditorInput;
import org.eclipse.ui.part.MultiPageEditorPart;
import org.json.simple.JSONObject;
import org.json.simple.parser.JSONParser;
import org.json.simple.parser.ParseException;
import org.json.simple.JSONArray;
import org.json.*;
import org.eclipse.ui.ide.IDE;

/**
 * How to create a multi-page editor.
 * This example has 3 pages:
 * <ul>
 * <li>page 0 contains a nested text editor.
 * <li>page 1 shows the Classification tab
 * <li>page 2 shows the Simulation tab
 * </ul>
 */
public class optimus extends MultiPageEditorPart implements IResourceChangeListener{

	/** The text editor used in page 0. */
	// Tecnalia20171024: I should find out how to assign a json editor
	public static TextEditor editor;
	private StyledText text;
	// create a json object
	public static JSONObject appjson;
	public static JSONArray appmicros;
	public static String newjson = "";
	public static String workingDir;
	/**
	 * Creates a multi-page editor example.
	 */
	public optimus() {
		super();
		
		ResourcesPlugin.getWorkspace().addResourceChangeListener(this);
		
		appjson = new JSONObject();
		appmicros = new JSONArray();
		
	}
	/**
	 * Creates page 0 of the multi-page editor,
	 * which contains a text editor.
	 */
	void createPage0() {
		try {

			editor = new TextEditor();
			int index = addPage(editor, getEditorInput());

			setPageText(index, editor.getTitle());
			
		} catch (PartInitException e) {
			ErrorDialog.openError(
				getSite().getShell(),
				"Error creating nested text editor",
				null,
				e.getStatus());
		}

	}
	// these lines of comments are a try of reading the JSON file and drawing the corresponding elements in classification tab
	// I leave it to try again for the second version - Mara Jos Lpe< - TECNALIA November 2017

//	private JSONObject initialfile() throws IOException, ParseException {
//		
//		String line;
//		FileReader f = new FileReader(workingDir);
//        BufferedReader b = new BufferedReader(f);
//       line = b.readLine();
//       JSONParser parser = new JSONParser();
//       JSONObject jsonfilecontent = (JSONObject) parser.parse(line);
//       //JSONObject jsonfilecontent = new JSONObject(line);
//       b.close();
//		return jsonfilecontent;
//	}
	/**
	 * Creates page 1 of the multi-page editor,
	 * which allows you to change the font used in page 2.
	 */
	// Tecnalia: The UI for the classification tool
	void createPage1() {

		Composite composite = new Classification(getContainer(), SWT.NONE);

		int index = addPage(composite);
		setPageText(index, "Classification");
	}
	/**
	 * Creates page 2 of the multi-page editor,
	 * which shows the sorted text.
	 */
	void createPage2() {
// Classification tab
		Composite composite = new Simulation (getContainer(), SWT.NONE);

		int index = addPage(composite);
		setPageText(index, "Simulation");
	}
	

	
	
	/**
	 * Creates the pages of the multi-page editor.
	 */
	protected void createPages() {
		createPage0();
		createPage1();
		createPage2();
	}
	/**
	 * The <code>MultiPageEditorPart</code> implementation of this 
	 * <code>IWorkbenchPart</code> method disposes all nested editors.
	 * Subclasses may extend.
	 */
	public void dispose() {
		ResourcesPlugin.getWorkspace().removeResourceChangeListener(this);
		super.dispose();
	}
	/**
	 * Saves the multi-page editor's document.
	 */
	public void doSave(IProgressMonitor monitor) {
		getEditor(0).doSave(monitor);

		
	}
	/**
	 * Saves the multi-page editor's document as another file.
	 * Also updates the text for page 0's tab, and updates this multi-page editor's input
	 * to correspond to the nested editor's.
	 */
	public void doSaveAs() {
		IEditorPart editor = getEditor(0);
		editor.doSaveAs();
		setPageText(0, editor.getTitle());
		setInput(editor.getEditorInput());
	}
	/* (non-Javadoc)
	 * Method declared on IEditorPart
	 */
	public void gotoMarker(IMarker marker) {
		setActivePage(0);
		IDE.gotoMarker(getEditor(0), marker);
	}
	/**
	 * The <code>MultiPageEditorExample</code> implementation of this method
	 * checks that the input is an instance of <code>IFileEditorInput</code>.
	 */
	public void init(IEditorSite site, IEditorInput editorInput)
		throws PartInitException {
		
		if (!(editorInput instanceof IFileEditorInput))
			throw new PartInitException("Invalid Input: Must be IFileEditorInput");
		workingDir = ResourcesPlugin.getWorkspace().getRoot().getLocation().toString();
		Integer position = editorInput.toString().indexOf("/");
		Integer length = editorInput.toString().length();
		workingDir = workingDir+editorInput.toString().substring(position,length-1);
		
		super.init(site, editorInput);
	}
	/* (non-Javadoc)
	 * Method declared on IEditorPart.
	 */
	public boolean isSaveAsAllowed() {
		return true;
	}
	/**
	 * Calculates the contents of page 0.
	 */
	protected void pageChange(int newPageIndex) {
		super.pageChange(newPageIndex);
		if (newPageIndex == 1) {
		}
		if (newPageIndex == 2) {
		}
		if (newPageIndex == 0) {
			// I do not know why but  I do not see the new content until the button save is selected.
			// to be analyzed for next version
			//IEditorPart editor = getEditor(0);
			//setPageText(0, editor.getTitle());
			//setInput(editor.getEditorInput());
		}
		}
	
	/**
	 * Closes all project files on project close.
	 */
	public void resourceChanged(final IResourceChangeEvent event){
		if(event.getType() == IResourceChangeEvent.PRE_CLOSE){
			Display.getDefault().asyncExec(new Runnable(){
				public void run(){
					IWorkbenchPage[] pages = getSite().getWorkbenchWindow().getPages();
					for (int i = 0; i<pages.length; i++){
						if(((FileEditorInput)editor.getEditorInput()).getFile().getProject().equals(event.getResource())){
							IEditorPart editorPart = pages[i].findEditor(editor.getEditorInput());
							pages[i].closeEditor(editorPart,true);
						}
					}
				}            
			});
		}
	}
}
